@@ -48,7 +48,7 @@ smst <- list(Method = Method, FixCoefMat = FixCoefMat, RandCoefMat = RandCoefMat
              converge = object$Converge, call = Call, ProfLogLik = object$ProfLogLik, devdf = object$dfReFe,
              LogLik = object$LogLik, varFix = object$varFix, varRanef = object$varRanef, link.disp = object$link.disp,
 			 likelihood = object$likelihood, call.rand.family = object$call.rand.family, 
-			 CAR.tau = object$CAR.tau, CAR.rho = object$CAR.rho)
+			 CAR.tau = object$CAR.tau, CAR.rho = object$CAR.rho, SAR.tau = object$SAR.tau, SAR.rho = object$SAR.rho)
   
 class(smst) <- "summary.hglm"
   
